"""
Checks whether we can call the hello endpoint.
"""
from unittest.mock import call

import pytest
from asynctest import CoroutineMock
from brewblox_service import scheduler

import brewblox_ispindel.__main__ as main

TESTED = main.__name__

BODY_OK = '{"name":"iSpindel000","ID":4974097,"angle":83.49442,"temperature":21.4375,"temp_units":"C",' + \
          '"battery":4.035453,"gravity":30.29128,"interval":60,"RSSI":-76}'

EVENT_OK = {"angle": 83.49442, "temperature": 21.4375, "battery": 4.035453, "gravity": 30.29128, "rssi": -76}


class DummyListener:

    def subscribe(self,
                  exchange_name=None,
                  routing=None,
                  exchange_type=None,
                  on_message=None,
                  ):
        self.exchange_name = exchange_name
        self.routing = routing
        self.exchange_type = exchange_type
        self.on_message = on_message


@pytest.fixture
async def app(app, mock_publisher, dummy_listener):
    app.router.add_routes(main.routes)
    scheduler.setup(app)
    return app


@pytest.fixture
def dummy_listener(mocker):
    m = mocker.patch(TESTED + '.events.get_listener')
    m.return_value = DummyListener()
    return m.return_value


@pytest.fixture
def mock_publisher(mocker):
    m = mocker.patch(TESTED + '.events.get_publisher')
    m.return_value.publish = CoroutineMock()
    return m.return_value


async def test_ispindel(app, client, mock_publisher):
    res = await client.post('/ispindel', data=BODY_OK)
    assert res.status == 200
    assert await res.text() == ''
    assert mock_publisher.publish.call_count > 0
    assert mock_publisher.publish.call_args_list[-1] == call(
        'brewcast',
        'iSpindel000',
        EVENT_OK)



from setuptools import find_packages, setup

setup(
    name='brewblox-ispindel',
    version='0.2.0',
    long_description=open('README.md').read(),
    url='https://github.com/bdelbosc/brewblox-ispindel',
    author='Benoit Delbosc',
    author_email='bdelbosc@free.fr',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Programming Language :: Python :: 3.7',
        'Intended Audience :: End Users/Desktop',
        'Topic :: System :: Hardware',
    ],
    keywords='brewing brewpi brewblox embedded plugin service ispindel hydrometer',
    packages=find_packages(exclude=['test', 'docker']),
    install_requires=[
        'brewblox-service'
    ],
    python_requires='>=3.7',
    extras_require={'dev': ['pipenv']}
)



"""
Example of how to import and use the brewblox service
"""
from aiohttp import web
from brewblox_service import brewblox_logger, events, scheduler, service

routes = web.RouteTableDef()
LOGGER = brewblox_logger(__name__)

# The brewblox history service is subscribed to this exchange by default
# Anything published here is logged
# See https://brewblox.netlify.com/dev/reference/event_logging.html for the spec
HISTORY_EXCHANGE = 'brewcast'


@routes.post('/ispindel')
async def ispindel_handler(request: web.Request) -> web.Response:
    """
    This endpoint accepts request from iSpindel when configured to use the Generic HTTP POST.

    ---
    tags:
    - iSpindel
    summary: Endpoint to receive iSpindel metrics.
    description: The iSpindel wake up and send an HTTP POST request to this endpoint.
    operationId: ispindel/ispindel
    produces:
    - text/plain
    parameters:
    -
        in: body a iSpindel JSON containing current metrics
        name: body
        description: Input message
        required: true
        schema:
            type: string
    """
    body = await request.json()
    name = body.get("name")
    temperature = body.get("temperature")
    battery = body.get("battery")
    gravity = body.get("gravity")
    rssi = body.get("RSSI")
    angle = body.get("angle")
    if not name or not temperature:
        LOGGER.info("Bad request: " + str(request.text()))
        return web.Response(status=400)
    publisher = events.get_publisher(request.app)
    await publisher.publish(HISTORY_EXCHANGE, name, {'temperature': temperature,
                                                     'battery': battery,
                                                     'angle': angle,
                                                     'rssi': rssi,
                                                     'gravity': gravity})
    LOGGER.info(f'iSpindel {name}, temp: {temperature}, gravity: {gravity}')
    return web.Response(status=200)


def add_events(app: web.Application):
    # Enable the task scheduler
    # This is required for the `events` feature
    scheduler.setup(app)

    # Enable event handling
    # Event subscription / publishing will be enabled after you call this function
    events.setup(app)


def main():
    app = service.create_app(default_name='ispindel')

    # Init events
    add_events(app)

    # Register routes
    app.router.add_routes(routes)

    # Add all default endpoints, and adds prefix to all endpoints
    #
    # Default endpoints are:
    # {prefix}/api/doc (Swagger documentation of endpoints)
    # {prefix}/_service/status (Health check: this endpoint is called to check service status)
    #
    # The prefix is automatically added for all endpoints. You don't have to do anything for this.
    # To change the prefix, you can use the --name command line argument.
    #
    # See brewblox_service.service for more details on how arguments are parsed.
    #
    # The default value is "YOUR_PACKAGE" (provided in service.create_app()).
    # This means you can now access the example/endpoint as "/YOUR_PACKAGE/example/endpoint"
    service.furnish(app)

    # service.run() will start serving clients async
    service.run(app)


if __name__ == '__main__':
    main()
